;
;*******************************************************;
;                                                       ;
;     256K PRINTER BUFFER PROGRAM USING THE 8xC451      ;
;        FOR CENTRONICS PARALLEL PRINTER PORTS          ;
;                                                       ;
;               SIGNETICS  CORPORATION                  ;
;                   October, 1988                       ;
;*******************************************************;
;
$Mod451
$Title(8XC451 Printer Buffer)
$Date(10/28/88)
;
; PORT USAGE:
;
; P0            Not used (reserved for data/address bus when external
;                 program memory is used).
; P1            Lower 8 bits of DRAM address (A0 - A7).
; P2            Not used (reserved for high-order address bus when external
;                 program memory is used).
;
; P3.0          (Reserved for serial port.)
; P3.1          (Reserved for serial port.)
; P3.2 (/INT0)  Input port strobe input (interrupt).
; P3.3 (/INT1)  Output port acknowledge input (interrupt).
; P3.4          Input port acknowledge output.
; P3.5          DRAM write enable output.
; P3.6 (/WR)    DRAM row address select output.
; P3.7 (/RD)    DRAM column address select output.
;
; P4.0          Upper bit of DRAM address (A8).
; P4.1          Reserved as an extra address line for 1 Megabit DRAMS.
; P4.2          Not used.
; P4.3          Output port busy input (OBUSY).
; P4.4-P4.7     Unused (not available on 64-pin DIP package).
;
; P5            DRAM output data.
;
; P6            Parallel output port.
; /IDS          Input port strobe input (ISTB).
; BFLAG         Input port busy output (IBUSY).
;
; AFLAG         Output port strobe output (OSTB).
; /ODS          Port 6 output enable, tied low.
;
;
; Internal Register/RAM Usage:
;
REFCNT  EQU     020h        ; Low order refresh byte.
;
; The following refer to the circular FIFO buffer
; implemented in the DRAM array.
;
INLOW   EQU     22h           ; Incoming address low byte.
INMID   EQU     23h           ; Incoming address mid byte.
INHI    EQU     24h           ; Incoming address high byte.
OUTLOW  EQU     25h           ; Outgoing address low byte.
OUTMID  EQU     26h           ; Outgoing address mid byte.
OUTHI   EQU     27h           ; Outgoing address high byte.
;
OACK    EQU     28h           ; Holds flag for output port acknowledge.
FOACK   BIT     OACK.0        ; Bit-address of output port acknowledge flag.
;
; Miscellaneous Equates:
;
TIME    EQU     -1000         ; Value for 1000 timer clocks = 1 millisecond.
TIMEHI  EQU     HIGH TIME     ; High byte of timer value.
TIMELO  EQU     LOW TIME      ; Low byte of timer value.
RAS     BIT     P3.6          ; DRAM column address select.
CAS     BIT     P3.7          ; DRAM row address select.
DRAMWR  BIT     P3.5          ; DRAM write control line.
IACK    BIT     P3.4          ; Input port ACK output.
ISTB    BIT     P3.2          ; Input port strobe line (INT0).
OBUSY   BIT     P4.3          ; Output port BUSY input.
OSTB    BIT     MA0           ; Output port strobe (MA0 bit in port 6 CSR).
;
;****************************************************************************
;
; Reset and Interrupt Jump Table
;
        ORG     00h           ; Power-on reset.
        AJMP    START
;
        ORG     03h           ; INT 0.
        AJMP    INDATA        ; Data at input port.
;
        ORG     0Bh           ; Timer 0.
        AJMP    REFRESH       ; Refresh DRAM array.
;
        ORG     13h           ; INT 1.
        AJMP    OPACK         ; Output port acknowledge.
;
;****************************************************************************
;
; Power up reset routine:
; Set up refresh timer, enable timer interrupt and
; external interrupt, initialize circular buffer pointers.
;
        ORG     18h
START:  MOV     SP,#40h       ; Initialize stack pointer.
        MOV     A,#00
        MOV     REFCNT,A      ; Initialize refresh counter.
        MOV     INLOW,A       ; Initialize FIFO pointers.
        MOV     INMID,A
        MOV     INHI,A
        MOV     OUTLOW,A
        MOV     OUTMID,A
        MOV     OUTHI,A
;
; Initialize interrupt priority register so that DRAM refresh
; (TF0) gets high priority, input port service (IE0) and output
; port acknowledge service get lower priority.  All other
; interrupts set to lower priority level.
;
        MOV     IE,#00000111b   ; Timer0, INT0, and INT1 enabled.
        MOV     IP,#00000010b   ; Timer0 high priority.
        MOV     TL0,#TIMELO
        MOV     TH0,#TIMEHI
        MOV     TMOD,#00000001b ; Operate Timer0 in mode 1.
        MOV     TCON,#00010101b ; Timer0 run, I0 and I1 = edge.
;
;
; Initialize Port 6 Control and Status Register.
;   - 'BFLAG' mode set to output value of IBF
;	     (input port BUSY signal : IBUSY)
;   - 'AFLAG' set as logic 1 output
;      (output port strobe signal : OSTB)
;   - 'IDS' active on negative level
;      (input port strobe signal : ISTB)
;
        MOV     CSR,#10011100b
        MOV     A,P6          ; Dummy read of P6 to clear IBF (IBUSY).
        SETB    EA            ; Enable interrupts.
;
;****************************************************************************
;
; Main Routine:
; Executes while not performing DRAM refresh or servicing
; input port interrupt.
;
; Check if buffer is not empty by comparing input and output
; pointers. If not empty, go to NOTMT to output a byte.
;
MAINLP: MOV     A,INLOW       ; Compare pointers.
        CJNE    A,OUTLOW,NOTMT
        MOV     A,INMID
        CJNE    A,OUTMID,NOTMT
        MOV     A,INHI
        CJNE    A,OUTHI,NOTMT
        SJMP    MAINLP
;
; Buffer is not empty: compute row & column addresses for
; a read cycle from DRAM.
;
NOTMT:  MOV     R4,OUTLOW     ; Save low byte of row.
        MOV     R5,OUTMID     ; Save upper bit of row.
        MOV     A,OUTHI       ; Shift to align correctly.
        RRC     A
        MOV     R7,A          ; Save upper column bit.
        MOV     A,OUTMID      ; Get low byte of column.
        RRC     A             ; Shift in bit from OUTHI.
        MOV     R6,A          ; Save.
;
; Now do actual DRAM access to get the data byte at computed
; address. Disable interrupts so we don't lose what we put
; out on the ports.
;
        CLR     EA            ; Disable interrupts.
        MOV     P1,R4         ; Low byte row address.
        MOV     A,R5          ; Get high byte row address.
        ORL     A,#0FEh       ; Make sure OBUSY stays high.
        MOV     P4,A
        CLR     RAS           ; /RAS low.
        MOV     P1,R6         ; Low byte column address.
        MOV     A,R7          ; High byte column address.
        ORL     A,#0FEh       ; Make sure OBUSY stays high.
        MOV     P4,A
        CLR     CAS           ; /CAS low.
        MOV     R4,P5         ; Get the data byte.
;
        SETB    CAS           ; /CAS high.
        SETB    RAS           ; /RAS high.
        CLR     FOACK         ; Clear acknowledge flag.
        SETB    EA            ; Re-enable interrupts.
;
PLOOP1: JB      OBUSY,PLOOP1  ; Loop if printer busy.
;
        CLR     EA            ; Disable interrupts.
        MOV     P6,R4         ; Move byte to output port.
        CLR     MA0           ; Assert output port strobe.
        NOP                   ; Kill some time.
        NOP
        NOP
        SETB    MA0           ; De-assert output port strobe.
        SETB    EA            ; Re-enable interrupts.
;
; Following waits for /ACK to occur on output port. Loops on
; acknowledge flag which is set by INT1 service routine when
; /ACK occurs.
;
PLOOP2: JNB     FOACK,PLOOP2  ; Wait till /ACK occurs.
;
        INC     OUTLOW        ; Increment output buffer pointer.
        MOV     A,OUTLOW
        CJNE    A,#00,PDONE
        INC     OUTMID
        MOV     A,OUTMID
        CJNE    A,#00,PDONE
        MOV     A,OUTHI
        INC     A
        ANL     A,#03h        ; Eliminate unused address bits
        MOV     OUTHI,A       ; and save.
;
; Check if input port busy flag was left asserted, indicating that
; the buffer was full after last input. If so, acknowledge input
; port and de-assert input busy signal.
;
PDONE:  JNB     IBF,MAINLP    ; Not busy, return to main loop.
        CLR     EA            ; Disable interrupts.
        CLR     IACK          ; Assert /IACK.
        NOP                   ; Wait 7 microseconds.
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        MOV     A,P6          ; Dummy read of P6 clears IBF (IBUSY).
        NOP                   ; Wait 5 microseconds.
        NOP
        NOP
        NOP
        NOP
        SETB    IACK          ; De-assert /IACK.
        SETB    EA            ; Re-enable interrupts.
        AJMP    MAINLP        ; Return to main loop.
;
;****************************************************************************
;
; Interrupt 1 Service Routine:
;
;   - Called when output port asserts /ACK.
;   - Sets FOACK flag and returns.
;
OPACK:  SETB    FOACK
        RETI
;
;****************************************************************************
;
; DRAM Refresh (Timer0) Interrupt Service:
;
;   - Called once every millisecond by timer interrupt.
;   - Refreshes 64 rows and then returns.
;   - Therefore refreshes all rows every 4 milliseconds.
;     (Note that 41256/51C256 DRAM only requires a 256 row refresh.)
;
REFRESH: PUSH   PSW
        MOV     TH0,#TIMEHI   ; Reload timer registers.
        MOV     TL0,#TIMELO
;
        MOV     P1,REFCNT     ; Get next row to refresh.
        MOVX    @R0,A         ; Pulse /RAS (/WR).
        INC     P1
        MOVX    @R0,A         ; 1
        INC     P1
        MOVX    @R0,A         ; 2
        INC     P1
        MOVX    @R0,A         ; 3
        INC     P1
        MOVX    @R0,A         ; 4
        INC     P1
        MOVX    @R0,A         ; 5
        INC     P1
        MOVX    @R0,A         ; 6
        INC     P1
        MOVX    @R0,A         ; 7
        INC     P1
        MOVX    @R0,A         ; 8
        INC     P1
        MOVX    @R0,A         ; 9
        INC     P1
        MOVX    @R0,A         ; 10
        INC     P1
        MOVX    @R0,A         ; 11
        INC     P1
        MOVX    @R0,A         ; 12
        INC     P1
        MOVX    @R0,A         ; 13
        INC     P1
        MOVX    @R0,A         ; 14
        INC     P1
        MOVX    @R0,A         ; 15
        INC     P1
        MOVX    @R0,A         ; 16
        INC     P1
        MOVX    @R0,A         ; 17
        INC     P1
        MOVX    @R0,A         ; 18
        INC     P1
        MOVX    @R0,A         ; 19
        INC     P1
        MOVX    @R0,A         ; 20
        INC     P1
        MOVX    @R0,A         ; 21
        INC     P1
        MOVX    @R0,A         ; 22
        INC     P1
        MOVX    @R0,A         ; 23
        INC     P1
        MOVX    @R0,A         ; 24
        INC     P1
        MOVX    @R0,A         ; 25
        INC     P1
        MOVX    @R0,A         ; 26
        INC     P1
        MOVX    @R0,A         ; 27
        INC     P1
        MOVX    @R0,A         ; 28
        INC     P1
        MOVX    @R0,A         ; 29
        INC     P1
        MOVX    @R0,A         ; 30
        INC     P1
        MOVX    @R0,A         ; 31
        INC     P1
        MOVX    @R0,A         ; 32
        INC     P1
        MOVX    @R0,A         ; 33
        INC     P1
        MOVX    @R0,A         ; 34
        INC     P1
        MOVX    @R0,A         ; 35
        INC     P1
        MOVX    @R0,A         ; 36
        INC     P1
        MOVX    @R0,A         ; 37
        INC     P1
        MOVX    @R0,A         ; 38
        INC     P1
        MOVX    @R0,A         ; 39
        INC     P1
        MOVX    @R0,A         ; 40
        INC     P1
        MOVX    @R0,A         ; 41
        INC     P1
        MOVX    @R0,A         ; 42
        INC     P1
        MOVX    @R0,A         ; 43
        INC     P1
        MOVX    @R0,A         ; 44
        INC     P1
        MOVX    @R0,A         ; 45
        INC     P1
        MOVX    @R0,A         ; 46
        INC     P1
        MOVX    @R0,A         ; 47
        INC     P1
        MOVX    @R0,A         ; 48
        INC     P1
        MOVX    @R0,A         ; 49
        INC     P1
        MOVX    @R0,A         ; 50
        INC     P1
        MOVX    @R0,A         ; 51
        INC     P1
        MOVX    @R0,A         ; 52
        INC     P1
        MOVX    @R0,A         ; 53
        INC     P1
        MOVX    @R0,A         ; 54
        INC     P1
        MOVX    @R0,A         ; 55
        INC     P1
        MOVX    @R0,A         ; 56
        INC     P1
        MOVX    @R0,A         ; 57
        INC     P1
        MOVX    @R0,A         ; 58
        INC     P1
        MOVX    @R0,A         ; 59
        INC     P1
        MOVX    @R0,A         ; 60
        INC     P1
        MOVX    @R0,A         ; 61
        INC     P1
        MOVX    @R0,A         ; 62
        INC     P1
        MOVX    @R0,A         ; 63
;
        INC     P1            ; Adjust for next time
        MOV     REFCNT,P1     ; and save.
        POP     PSW
        RETI
;
;****************************************************************************
;
; Data at Input Port:
;
; This routine is called via interrupt INT0 whenever data
; is strobed into the input port. It saves the data into the
; DRAM array and increments the input pointer. If the output
; pointer is now equal to the input pointer, then the buffer
; is full, and we leave the busy flag set so that no more
; data can be input until some is output and the buffer is
; no longer full.
;
;
INDATA: PUSH    PSW
        PUSH    ACC
        MOV     R1,INLOW      ; Lower 8 bits of row to R1.
        MOV     R2,INMID      ; Upper bit of row to R2.
        MOV     A,INHI        ; Get upper 2 bits.
        RRC     A             ; LSB to carry.
        MOV     R0,A
        MOV     A,INMID
        RRC     A             ; Shift bit into MSB.
        MOV     R3,A          ; Save.
;
        CLR     EA            ; Disable interrupts.
        MOV     P1,R1         ; LSB row address.
        MOV     A,R2          ; MSB row address.
        ORL     A,#0FEh       ; Make sure OBUSY stays high.
        MOV     P4,A          ; MSB row address.
STBLP:  JNB     ISTB,STBLP    ; Check for end of strobe before DRAM write.
        CLR     RAS           ; /RAS low.
        CLR     DRAMWR        ; /WR low.
        MOV     P1,R3         ; LSB column address.
        MOV     A,R0          ; MSB column address.
        ORL     A,#0FEh       ; Make sure OBUSY stays high.
        MOV     P4,A          ; MSB column address.
        MOVX    A,@R0         ; Pulse /CAS low.
        SETB    RAS           ; /RAS high.
        SETB    DRAMWR        ; /WR high.
        SETB    EA            ; Re-enable interrupts.
;
        INC     INLOW         ; Increment input buffer pointer.
        MOV     A,INLOW
        CJNE    A,#00,CKFULL
        INC     INMID
        MOV     A,INMID
        CJNE    A,#00,CKFULL
        MOV     A,INHI
        INC     A
        ANL     A,#03h        ; Eliminate unused address bits.
        MOV     INHI,A
;
; Compare input pointer to output pointer to see if the buffer is full.
;
CKFULL: MOV     A,INLOW
        CJNE    A,OUTLOW,INCLR
        MOV     A,INMID
        CJNE    A,OUTMID,INCLR
        MOV     A,INHI
        CJNE    A,OUTHI,INCLR
;
; If we get here, the buffer is full, so skip the acknowledge pulse.
;
        SJMP    INDONE
;
;
; Send acknowledge pulse on /IACK line for 7 microseconds,
; de-assert input BUSY signal halfway through.
;
INCLR:  CLR     EA            ; Disable interrupts.
        CLR     IACK          ; Assert /IACK.
        NOP                   ; Wait 7 microseconds.
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        MOV     A,P6          ; Dummy read of P6 clears IBF (IBUSY).
        NOP                   ; Wait 5 microseconds before clearing /IACK.
INDONE: POP     ACC
        POP     PSW
        SETB    IACK          ; De-assert /IACK.
        SETB    EA            ; Re-enable interrupts.
        RETI
;
        END
